
'use strict';

const { Gio, GLib } = imports.gi;
const Main = imports.ui.main;

const ByteArray = imports.byteArray;

const docker_commands_to_labels = {
	start: 'Start',
	"start i": 'Start interactive',
	restart: 'Restart',
	stop: 'Stop',
	pause: 'Pause',
	unpause: 'Unpause',
	rm: 'Remove',
	exec: 'Exec'
};

/**
 * Check if docker is installed
 * @return {Boolean} whether docker is installed or not
 */
function is_docker_installed() {
	return !!GLib.find_program_in_path('docker');
}

/**
 * Return containers name and status.
 * @return {Array.<{name: String, status: String}>} every object represents a container
 */
async function get_containers() {
	let command = `docker container list --all --format "{{.Names}};{{.State}}"`;

	let [, c ] = GLib.shell_parse_argv(command);

	let out = "";
	await exec_communicate(c)
		.then((output) => {out = output})
		.catch((err) => {
			Main.notify(`Docker: ${err}`);
		});

	if (!out.length) {
		return [];
	}

	// Found containers
	return out
			.replaceAll("\"","")
			.split("\n")
			.filter(element => element)
			.map(str => str.split(";"))
			.map(s => {
				return {
					name:s[0],
					status:s[1]
				};
			});

}

/**
 * Run docker command.
 * @param {String} command - docker command
 * @param {String} name - name of container
 */
function run_command(command, name, cancellable = null) {
	let c = "";
	switch (command){
		case "exec":
			c = `gnome-terminal -- sh -c 'docker exec -it ${name} bash' `

			GLib.spawn_command_line_async(c);
		return;

		case "start i":
			c = `gnome-terminal -- sh -c 'docker start -i ${name}; exec $SHELL' `

			GLib.spawn_command_line_async(c);
		return;

		default:
			c = `docker ${command} ${name}`
	}

	let proc = Gio.Subprocess.new(
		c.split(" "),
		Gio.SubprocessFlags.STDOUT_PIPE | Gio.SubprocessFlags.STDERR_PIPE
	);


	proc.communicate_utf8_async(null, null, (proc, res) => {
		try {
			let [, , stderr] = proc.communicate_utf8_finish(res);

			if (!proc.get_successful()) {
				throw new Error(stderr);
			}

			Main.notify(`${docker_commands_to_labels[command]} container: ${name}`);
		} catch (e) {
			Main.notifyError('Docker ERROR', `${e}`);
		}
	});
}

/**
 * Execute a command asynchronously and return the output from `stdout` on
 * success or throw an error with output from `stderr` on failure.
 *
 * If given, @input will be passed to `stdin` and @cancellable can be used to
 * stop the process before it finishes.
 *
 * @param {string[]} argv - a list of string arguments
 * @param {string} [input] - Input to write to `stdin` or %null to ignore
 * @param {Gio.Cancellable} [cancellable] - optional cancellable object
 * @returns {Promise<string>} - The process output
 */
async function exec_communicate(argv, input = null, cancellable = null) {
	let cancelId = 0;
	let flags = (Gio.SubprocessFlags.STDOUT_PIPE |
				 Gio.SubprocessFlags.STDERR_PIPE);

	if (input !== null)
		flags |= Gio.SubprocessFlags.STDIN_PIPE;

	let proc = new Gio.Subprocess({
		argv: argv,
		flags: flags
	});
	proc.init(cancellable);

	if (cancellable instanceof Gio.Cancellable) {
		cancelId = cancellable.connect(() => proc.force_exit());
	}

	return new Promise((resolve, reject) => {
		proc.communicate_utf8_async(input, null, (proc, res) => {
			try {
				let [, stdout, stderr] = proc.communicate_utf8_finish(res);
				let status = proc.get_exit_status();

				if (status !== 0) {
					throw new Gio.IOErrorEnum({
						code: Gio.io_error_from_errno(status),
						message: stderr ? stderr.trim() : GLib.strerror(status)
					});
				}

				resolve(stdout.trim());
			} catch (e) {
				reject(e);
			} finally {
				if (cancelId > 0) {
					cancellable.disconnect(cancelId);
				}
			}
		});
	});
}
